from django.core import validators
from django.db import models

from django.utils.functional import cached_property


class VRPaymentDecimalField(models.DecimalField):
    """
    Decimalfield declared as described in https://vr-pay-ecommerce.docs.oppwa.com/reference/parameters#basic
    """

    def __init__(
        self, verbose_name=None, name=None, max_digits=12, decimal_places=2, **kwargs
    ):
        super().__init__(verbose_name, name, max_digits, decimal_places, **kwargs)

    @cached_property
    def validators(self):
        return super().validators + [
            validators.RegexValidator(regex="[0-9]{1,10}(\.[0-9]{2})?")
        ]



from django.db import models


class BaseModel(models.Model):
    created_at = models.DateTimeField("Created at", auto_now_add=True)
    last_modified = models.DateTimeField("Last modified", auto_now=True)

    class Meta:
        abstract = True



import json

from django.db.models import Q, QuerySet
from django.db.models.manager import Manager

from .utils.transaction_status import (
    TRANSACTION_SUCCESSFULLY_PROCESSED_REGEX,
    TRANSACTION_SUCCESSFULLY_PROCESSED_NEEDS_REVIEW_REGEX,
    TRANSACTION_PENDING_REGEX,
    TRANSACTION_PENDING_MIGHT_CHANGE_EXTERNALLY_REGEX,
)


class VRPaymentBasicPaymentManager(Manager):
    def get(self, *args, **kwargs):
        return super().select_related("checkout_response").get(*args, **kwargs)

    def filter(self, *args, **kwargs):
        return super().select_related("checkout_response").filter(*args, **kwargs)


class VRPaymentAPIResponseManger(Manager):
    def create_from_response(
        self, response, basic_payment=None,
    ):
        response_json = response.json()
        if "payments" in response_json:
            # querying the transaction status can return several payments, but we currently only support one
            assert len(response_json["payments"]) == 1, "too many payments in response"
            response_json.update(response_json["payments"].pop())
        vr_response = self.model(
            basic_payment=basic_payment,
            http_status_code=response.status_code,
            url=response.url,
            raw_headers=json.dumps(dict(response.headers)),
            raw_content=response.json(),  # response_json might have been altered already; save the raw json!
            build_number=response_json.get("buildNumber"),
            ndc=response_json.get("ndc"),
            vr_pay_id=response_json.get("id"),
            reference_id=response_json.get("referencedId"),
            payment_brand=response_json.get("paymentBrand"),
            amount=response_json.get("amount"),
            currency=response_json.get("currency"),
            descriptor=response_json.get("descriptor"),
            result_code=response_json["result"].get("code"),
            result_description=response_json["result"].get("description"),
            result_avs_response=response_json["result"].get("avsResponse"),
            result_cvv_response=response_json["result"].get("cvvResponse"),
            result_details=response_json.get("resultDetails"),
            result_details_acquirer_response=response_json["resultDetails"].get(
                "AcquirerResponse"
            )
            if "resultDetails" in response_json
            else None,
            card_bin=response_json["card"].get("bin")
            if "card " in response_json
            else None,
            card_holder=response_json["card"].get("holder")
            if "card " in response_json
            else None,
            card_expiry_month=response_json["card"].get("expiryMonth")
            if "card " in response_json
            else None,
            card_expiry_year=response_json["card"].get("expiryYear")
            if "card " in response_json
            else None,
            merchant_bank_account_holder=response_json["merchant"]["bankAccount"].get(
                "holder"
            )
            if "merchant" in response_json
            and "bankAccount" in response_json["merchant"]
            else None,
            merchant_bank_account_nummber=response_json["merchant"]["bankAccount"].get(
                "nummber"
            )
            if "merchant" in response_json
            and "bankAccount" in response_json["merchant"]
            else None,
            merchant_bank_account_bic=response_json["merchant"]["bankAccount"].get(
                "bic"
            )
            if "merchant" in response_json
            and "bankAccount" in response_json["merchant"]
            else None,
            merchant_bank_account_country=response_json["merchant"]["bankAccount"].get(
                "country"
            )
            if "merchant" in response_json
            and "bankAccount" in response_json["merchant"]
            else None,
            risk_score=response_json["risk"].get("score")
            if "risk" in response_json
            else None,
            other=response_json.get("Other"),
        )
        vr_response.save()
        return vr_response

    def filter_successfully_processed_all(self) -> QuerySet:
        return self.filter(
            Q(result_code__regex=TRANSACTION_SUCCESSFULLY_PROCESSED_REGEX)
            | Q(
                result_code__regex=TRANSACTION_SUCCESSFULLY_PROCESSED_NEEDS_REVIEW_REGEX
            )
        )

    def filter_successfully_processed(self) -> QuerySet:
        return self.filter(result_code__regex=TRANSACTION_SUCCESSFULLY_PROCESSED_REGEX)

    def filter_successfully_processed_needs_review(self) -> QuerySet:
        return self.filter(
            result_code__regex=TRANSACTION_SUCCESSFULLY_PROCESSED_NEEDS_REVIEW_REGEX
        )

    def filter_pending_all(self) -> QuerySet:
        return self.filter(
            Q(result_code__regex=TRANSACTION_PENDING_REGEX)
            | Q(result_code__regex=TRANSACTION_PENDING_MIGHT_CHANGE_EXTERNALLY_REGEX)
        )

    def filter_pending(self) -> QuerySet:
        return self.filter(result_code__regex=TRANSACTION_PENDING_REGEX)

    def filter_pending_might_change(self) -> QuerySet:
        return self.filter(
            result_code__regex=TRANSACTION_PENDING_MIGHT_CHANGE_EXTERNALLY_REGEX
        )



import logging

import requests

from ..models import VRPaymentBasicPayment, VRPaymentBasicPaymentStatusResponse

logger = logging.getLogger(__name__)


class TransactionWrapper(object):
    """
    https://vr-pay-ecommerce.docs.oppwa.com/tutorials/reporting/transaction
    """

    def _get_transaction_status(
        self, url: str, basic_payment: VRPaymentBasicPayment
    ) -> VRPaymentBasicPaymentStatusResponse:
        try:
            response = self._call_api(url, "GET")
            response.raise_for_status()
        except requests.HTTPError as e:
            # log error, save error
            logger.error(e)
        return VRPaymentBasicPaymentStatusResponse.objects.create_from_response(
            response, basic_payment=basic_payment
        )

    def get_transaction_by_payment_id(
        self, basic_payment: VRPaymentBasicPayment
    ) -> VRPaymentBasicPaymentStatusResponse:
        url = (
            f"v1/query/{getattr(basic_payment, 'payment_id')}?entityId={self.entity_id}"
        )
        return self._get_transaction_status(url, basic_payment)

    def get_transaction_by_merchant_transaction_id(
        self, basic_payment: VRPaymentBasicPayment
    ) -> VRPaymentBasicPaymentStatusResponse:
        url = f"v1/query?entityId={self.entity_id}&merchantTransactionId={getattr(basic_payment, 'merchant_transaction_id')}"
        return self._get_transaction_status(url, basic_payment)



import re

"""
checks for VR Payment result codes

see https://vr-pay-ecommerce.docs.oppwa.com/reference/resultCodes for full list
"""

TRANSACTION_SUCCESSFULLY_PROCESSED_REGEX = r"^(000\.000\.|000\.100\.1|000\.[36])"
TRANSACTION_SUCCESSFULLY_PROCESSED_NEEDS_REVIEW_REGEX = (
    r"^(000\.400\.0[^3]|000\.400\.100)"
)
TRANSACTION_PENDING_REGEX = r"^(000\.200)"
TRANSACTION_PENDING_MIGHT_CHANGE_EXTERNALLY_REGEX = r"^(800\.400\.5|100\.400\.500)"


def check_transaction_status(regex, result_code: str) -> bool:
    return True if re.search(regex, result_code) else False


def check_transaction_successful(result_code: str) -> bool:
    """
    shortcut to see if the transaction was successful.
    NOTE: might be best to check separately for check_transaction_successfully_processed_needs_review
    """
    return check_transaction_successfully_processed(
        result_code
    ) or check_transaction_successfully_processed_needs_review(result_code)


def check_transaction_pending(result_code: str) -> bool:
    """
    shortcut to see if the transaction is pending.
    """
    return check_transaction_pending(
        result_code
    ) or check_transaction_pending_might_change_externally(result_code)


def check_transaction_rejected(result_code: str) -> bool:
    """
    shortcut to see if the transaction was rejected.
    """
    return (
        check_transaction_rejected_3dsecure_risk(result_code)
        or check_transaction_rejected_bank(result_code)
        or check_transaction_rejected_communications_error(result_code)
        or check_transaction_rejected_systems_error(result_code)
        or check_transaction_rejected_async_error(result_code)
        or check_transaction_rejected_soft_decline(result_code)
        or check_transaction_rejected_risk_handling_external_risk_system(result_code)
        or check_transaction_rejected_risk_handling_address_validation(result_code)
        or check_transaction_rejected_risk_handling_3dsecure(result_code)
        or check_transaction_rejected_risk_handling_blacklist_validation(result_code)
        or check_transaction_rejected_risk_handling_risk_validation(result_code)
        or check_transaction_rejected_configuration_validation(result_code)
        or check_transaction_rejected_registration_validation(result_code)
        or check_transaction_rejected_job_validation(result_code)
        or check_transaction_rejected_reference_validation(result_code)
        or check_transaction_rejected_format_validation(result_code)
        or check_transaction_rejected_address_validation(result_code)
        or check_transaction_rejected_contact_validation(result_code)
        or check_transaction_rejected_account_validation(result_code)
        or check_transaction_rejected_amount_validation(result_code)
        or check_transaction_rejected_risk_management(result_code)
    )


########################################################
#                                                      #
# Result codes for successful and pending transactions #
#                                                      #
########################################################


def check_transaction_successfully_processed(result_code: str) -> bool:
    """
    Result codes for successfully processed transactions

    The regular expression pattern for filtering out this group is: /^(000\.000\.|000\.100\.1|000\.[36])/
    """
    return check_transaction_status(
        TRANSACTION_SUCCESSFULLY_PROCESSED_REGEX, result_code
    )


def check_transaction_successfully_processed_needs_review(result_code: str) -> bool:
    """
    Result codes for successfully processed transactions that should be manually reviewed

    The regular expression pattern for filtering out this group is: /^(000\.400\.0[^3]|000\.400\.100)/

    """
    return check_transaction_status(
        TRANSACTION_SUCCESSFULLY_PROCESSED_NEEDS_REVIEW_REGEX, result_code
    )


def check_transaction_pending(result_code: str) -> bool:
    """
    Result codes for pending transactions

    The regular expression pattern for filtering out this group is: /^(000\.200)/. These codes mean that there is an open session in the background, meaning within half an hour there will be a status change, if nothing else happens, to timeout.
    """
    return check_transaction_status(TRANSACTION_PENDING_REGEX, result_code)


def check_transaction_pending_might_change_externally(result_code: str) -> bool:
    """
    Result codes for pending transactions

    There is another kind of pending regular expression pattern for filtering out this group is: /^(800\.400\.5|100\.400\.500)/. These codes describe a situation where the status of a transaction can change even after several days."""
    return check_transaction_status(
        TRANSACTION_PENDING_MIGHT_CHANGE_EXTERNALLY_REGEX, result_code
    )


##########################################
#                                        #
# Result codes for rejected transactions #
#                                        #
##########################################


def check_transaction_rejected_3dsecure_risk(result_code: str) -> bool:
    """
    Result codes for rejections due to 3Dsecure and Intercard risk checks

    The regular expression pattern for filtering out this group is: /^(000\.400\.[1][0-9][1-9]|000\.400\.2)/
    """
    regex = r"^(000\.400\.[1][0-9][1-9]|000\.400\.2)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_bank(result_code: str) -> bool:
    """
    Result codes for rejections by the external bank or similar payment system

    The regular expression pattern for filtering out this group is: /^(800\.[17]00|800\.800\.[123])/
    """
    regex = r"^(800\.[17]00|800\.800\.[123])"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_communications_error(result_code: str) -> bool:
    """
    Result codes for rejections due to communication errors

    The regular expression pattern for filtering out this group is: /^(900\.[1234]00|000\.400\.030)/
    """
    regex = r"^(900\.[1234]00|000\.400\.030)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_systems_error(result_code: str) -> bool:
    """
    Result codes for rejections due to system errors

    The regular expression pattern for filtering out this group is: /^(800\.[56]|999\.|600\.1|800\.800\.[84])/
    """
    regex = r"^(800\.[56]|999\.|600\.1|800\.800\.[84])"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_async_error(result_code: str) -> bool:
    """
    Result codes for rejections due to error in asynchonous workflow

    The regular expression pattern for filtering out this group is: /^(100\.39[765])/
    """
    regex = r"^(100\.39[765])"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_soft_decline(result_code: str) -> bool:
    """
    Result codes for Soft Declines

    The regular expression pattern for filtering out this group is: /^(300\.100\.100)/
    """
    regex = r"^(300\.100\.100)"
    return check_transaction_status(regex, result_code)


########################################
#                                      #
# Rejections specific to risk handling #
#                                      #
########################################


def check_transaction_rejected_risk_handling_external_risk_system(
    result_code: str,
) -> bool:
    """
    Result codes for rejections due to checks by external risk systems

    The regular expression pattern for filtering out this group is: /^(100\.400\.[0-3]|100\.38|100\.370\.100|100\.370\.11)/
    """
    regex = r"^(100\.400\.[0-3]|100\.38|100\.370\.100|100\.370\.11)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_risk_handling_address_validation(
    result_code: str,
) -> bool:
    """
    Result codes for rejections due to address validation

    The regular expression pattern for filtering out this group is: /^(800\.400\.1)/
    """
    regex = r"^(800\.400\.1)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_risk_handling_3dsecure(result_code: str) -> bool:
    """
    Result codes for rejections due to 3Dsecure

    The regular expression pattern for filtering out this group is: /^(800\.400\.2|100\.380\.4|100\.390)/
    """
    regex = r"^(800\.400\.2|100\.380\.4|100\.390)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_risk_handling_blacklist_validation(
    result_code: str,
) -> bool:
    """
    Result codes for rejections due to blacklist validation

    The regular expression pattern for filtering out this group is: /^(100\.100\.701|800\.[32])/
    """
    regex = r"^(100\.100\.701|800\.[32])"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_risk_handling_risk_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to risk validation

    The regular expression pattern for filtering out this group is: /^(800\.1[123456]0)/
    """
    regex = r"^(800\.1[123456]0)"
    return check_transaction_status(regex, result_code)


#################################################
#                                               #
# Result codes for rejections due to validation #
#                                               #
#################################################


def check_transaction_rejected_configuration_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to configuration validation

    The regular expression pattern for filtering out this group is: /^(600\.[23]|500\.[12]|800\.121)/
    """
    regex = r"^(600\.[23]|500\.[12]|800\.121)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_registration_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to registration validation

    The regular expression pattern for filtering out this group is: /^(100\.[13]50)/
    """
    regex = r"^(100\.[13]50)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_job_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to job validation

    The regular expression pattern for filtering out this group is: /^(100\.250|100\.360)/
    """
    regex = r"^(100\.250|100\.360)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_reference_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to reference validation

    The regular expression pattern for filtering out this group is: /^(700\.[1345][05]0)/
    """
    regex = r"^(700\.[1345][05]0)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_format_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to format validation

    The regular expression pattern for filtering out this group is: /^(200\.[123]|100\.[53][07]|800\.900|100\.[69]00\.500)/
    """
    regex = r"^(200\.[123]|100\.[53][07]|800\.900|100\.[69]00\.500)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_address_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to address validation

    The regular expression pattern for filtering out this group is: /^(100\.800)/
    """
    regex = r"^(100\.800)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_contact_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to contact validation

    The regular expression pattern for filtering out this group is: /^(100\.[97]00)/
    """
    regex = r"^(100\.[97]00)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_account_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to account validation

    The regular expression pattern for filtering out this group is: /^(100\.100|100.2[01])/
    """
    regex = r"^(100\.100|100.2[01])"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_amount_validation(result_code: str) -> bool:
    """
    Result codes for rejections due to amount validation

    The regular expression pattern for filtering out this group is: /^(100\.55)/
    """
    regex = r"^(100\.55)"
    return check_transaction_status(regex, result_code)


def check_transaction_rejected_risk_management(result_code: str) -> bool:
    """
    Result codes for rejections due to risk management

    The regular expression pattern for filtering out this group is: /^(100\.380\.[23]|100\.380\.101)/
    """
    regex = r"^(100\.380\.[23]|100\.380\.101)"
    return check_transaction_status(regex, result_code)


###################################
#                                 #
# Chargeback related result codes #
#                                 #
###################################


def check_chargeback_related(result_code: str) -> bool:
    """
    Chargeback related result codes

    The regular expression pattern for filtering out this group is: /^(000\.100\.2)/
    """
    regex = r"^(000\.100\.2)"
    return check_transaction_status(regex, result_code)



# Create your views here.

from django.http import (
    HttpResponseBadRequest,
    Http404,
    HttpResponseRedirect,
)
from django.urls import reverse
from django.utils.datastructures import MultiValueDictKeyError
from django.views import View
from django.views.generic import TemplateView

from . import settings
from .models import VRPaymentBasicPayment
from .wrapper import VRPaymentWrapper


class VRPaymentBasicCheckoutView(TemplateView):
    template_name = "vr_pay/checkout.html"

    def get(self, request, *args, **kwargs):
        basic_payment = VRPaymentBasicPayment.objects.get(
            merchant_transaction_id=kwargs.get("merchant_transaction_id")
        )
        if basic_payment.payment_responses.exists():
            return HttpResponseBadRequest(
                "This merchant_transaction_id has already been used."
            )
        kwargs.update({"basic_payment": basic_payment})
        return super(VRPaymentBasicCheckoutView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        basic_payment = kwargs.get("basic_payment")
        context.update(
            {
                "basic_payment": basic_payment,
                "vr_pay_src_url": settings.VR_PAYMENT_TEST_URL
                if basic_payment.sandbox
                else settings.VR_PAYMENT_LIVE_URL,
                "shopper_result_url": self.request.build_absolute_uri(
                    reverse(settings.VR_PAYMENT_SHOPPER_RESULT_URL_NAME)
                ),
            }
        )
        return context


class VRPaymentReturnView(View):
    basic_payment = None

    def get_error_url(self):
        return reverse(settings.VR_PAYMENT_ERROR_URL_NAME)

    def get_rejected_url(self):
        return reverse(settings.VR_PAYMENT_ERROR_URL_NAME)

    def get_pending_url(self):
        return reverse(settings.VR_PAYMENT_PENDING_URL_NAME)

    def get_success_url(self):
        return reverse(settings.VR_PAYMENT_SUCCESS_URL_NAME)

    def get_redirect_url(self, entity_id: str = None, bearer_token: str = None, sandbox: bool = None):
        redirect_url = None
        # check if already successful
        latest_payment_response = self.basic_payment.payment_responses.last()
        if latest_payment_response:
            redirect_url = (
                self.get_success_url()
                if latest_payment_response
                   == self.basic_payment.payment_responses.filter_successfully_processed().last()
                else None
            )

        if not redirect_url:
            # check status from VR Pay
            vr_payment_wrapper = VRPaymentWrapper(entity_id=entity_id, bearer_token=bearer_token, sandbox=sandbox)
            vr_payment_status = vr_payment_wrapper.get_checkout_status(basic_payment=self.basic_payment)
            if vr_payment_status.is_successful:
                self.basic_payment.payment_id = vr_payment_status.vr_pay_id
                redirect_url = self.get_success_url()
            elif vr_payment_status.is_rejected:
                redirect_url = self.get_rejected_url()
            elif vr_payment_status.is_pending:
                redirect_url = self.get_pending_url()
            else:
                redirect_url = self.get_error_url()
        return redirect_url

    def get(self, *args, **kwargs):
        update_fields = []
        try:
            self.basic_payment = VRPaymentBasicPayment.objects.get(
                checkout_response__vr_pay_id=self.request.GET["id"]
            )
            if self.basic_payment.resource_path:
                if self.basic_payment.resource_path != self.request.GET["resourcePath"]:
                    raise AssertionError(
                        f"resource_path for checkout_id: {self.request.GET['id']} changed to '{self.request.GET['resourcePath']}'"
                    )
            else:
                self.basic_payment.resource_path = self.request.GET["resourcePath"]
                update_fields.append("resource_path")
        except VRPaymentBasicPayment.DoesNotExist as e:
            raise Http404(e)
        except MultiValueDictKeyError as e:
            return HttpResponseBadRequest(f"expected keyword {e} not found")

        update_fields.append("payment_id")
        self.basic_payment.save(update_fields=update_fields)
        return HttpResponseRedirect(self.get_redirect_url())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.urls import path
from django.views.generic import TemplateView

from . import views

app_name = "vr_pay"


urlpatterns = [
    path(
        "checkout/<str:merchant_transaction_id>/",
        views.VRPaymentBasicCheckoutView.as_view(),
        name="checkout",
    ),
    path("return/", views.VRPaymentReturnView.as_view(), name="return"),
    path(
        "status/pending/",
        TemplateView.as_view(
            template_name="vr_pay/status.html", extra_context={"status": "pending"}
        ),
        name="status-pending",
    ),
    path(
        "status/rejected/",
        TemplateView.as_view(
            template_name="vr_pay/status.html", extra_context={"status": "rejected"}
        ),
        name="status-rejected",
    ),
    path(
        "status/success/",
        TemplateView.as_view(
            template_name="vr_pay/status.html", extra_context={"status": "success"}
        ),
        name="status-success",
    ),
    path(
        "status/error/",
        TemplateView.as_view(
            template_name="vr_pay/status.html", extra_context={"status": "error"}
        ),
        name="status-error",
    ),
]



from .payment import (
    VRPaymentBasicPayment,
    VRPaymentBasicPaymentStatusResponse,
    VRPaymentCheckoutResponse,
)

__all__ = [
    "VRPaymentBasicPayment",
    "VRPaymentBasicPaymentStatusResponse",
    "VRPaymentCheckoutResponse",
]



from django.conf import settings

DJANGO_VR_PAYMENT_VERSION = "0.0.1"

# VR Payment Settings
VR_PAYMENT_BEARER_TOKEN = getattr(
    settings,
    "VR_PAYMENT_BEARER_TOKEN",
    "OGE4Mjk0MTc0ZTczNWQwYzAxNGU3OGJlYjZjNTE1NGZ8Y1RaakFtOWM4Nw==",
)
VR_PAYMENT_ENTITY_ID = getattr(
    settings, "VR_PAYMENT_ENTITY_ID", "8a8294174e735d0c014e78beb6b9154b"
)

VR_PAYMENT_SANDBOX = getattr(settings, "VR_PAYMENT_SANDBOX", True)
VR_PAYMENT_TEST_URL = getattr(
    settings, "VR_PAYMENT_TEST_URL", "https://test.vr-pay-ecommerce.de/"
)
VR_PAYMENT_LIVE_URL = getattr(
    settings, "VR_PAYMENT_LIVE_URL", "https://vr-pay-ecommerce.de/"
)


# Internal Settings
VR_PAYMENT_SHOPPER_RESULT_URL_NAME = getattr(
    settings, "VR_PAYMENT_SHOPPER_RESULT_URL_NAME", "vr-payment:return"
)
VR_PAYMENT_ERROR_URL_NAME = getattr(
    settings, "VR_PAYMENT_ERROR_URL_NAME", "vr-payment:status-error"
)
VR_PAYMENT_REJECTED_URL_NAME = getattr(
    settings, "VR_PAYMENT_REJECTED_URL_NAME", "vr-payment:status-rejected"
)
VR_PAYMENT_PENDING_URL_NAME = getattr(
    settings, "VR_PAYMENT_PENDING_URL_NAME", "vr-payment:status-pending"
)
VR_PAYMENT_SUCCESS_URL_NAME = getattr(
    settings, "VR_PAYMENT_SUCCESS_URL_NAME", "vr-payment:status-success"
)



# Generated by Django 2.2.10 on 2020-07-10 23:29

import django.contrib.postgres.fields.jsonb
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_vr_pay.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="VRPaymentBasicPayment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created at"),
                ),
                (
                    "last_modified",
                    models.DateTimeField(auto_now=True, verbose_name="Last modified"),
                ),
                (
                    "entity_id",
                    models.CharField(
                        help_text="The entity required to authorize the request. This should be the channel entity identifier. In case channel dispatching is activated then it should be the merchant entity identifier.",
                        max_length=32,
                        verbose_name="Entity ID",
                    ),
                ),
                (
                    "amount",
                    django_vr_pay.fields.VRPaymentDecimalField(
                        decimal_places=2,
                        help_text="Indicates the amount of the payment request. The dot is used as decimal separator. The amount is the only amount value which is processing relevant. All other amount declarations like taxAmount or shipping.cost are already included.",
                        max_digits=12,
                        verbose_name="Amount",
                    ),
                ),
                (
                    "tax_amount",
                    django_vr_pay.fields.VRPaymentDecimalField(
                        decimal_places=2,
                        help_text="Indicates the tax amount of the payment request.",
                        max_digits=12,
                        verbose_name="Tax amount",
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        default="EUR",
                        help_text="The ISO 4217 currency code of the payment request's amount",
                        max_length=3,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "payment_brand",
                    models.CharField(
                        blank=True,
                        help_text="The brand specifies the method of payment for the request. This is optional if you want to use brand detection for credit cards, if not then it is mandatory.",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                regex="[a-zA-Z0-9_]{1,32}"
                            )
                        ],
                        verbose_name="Payment brand",
                    ),
                ),
                (
                    "payment_type",
                    models.CharField(
                        choices=[
                            ("PA", "Preauthorization"),
                            ("DB", "Debit"),
                            ("CD", "Credit"),
                            ("CP", "Capture"),
                            ("RV", "Reversal"),
                            ("RF", "Refund"),
                        ],
                        help_text="The payment type for the request. See docs for supported types",
                        max_length=2,
                        verbose_name="Payment type",
                    ),
                ),
                (
                    "descriptor",
                    models.CharField(
                        blank=True,
                        help_text="Can be used to populate all or part of the Merchant Name descriptor, which often appears on the first line of the shopper's statement. The full use of this field depends on the Merchant Account configuration. NOTE: merchant.name can override any data sent in this field.",
                        max_length=127,
                        null=True,
                        verbose_name="Descriptor",
                    ),
                ),
                (
                    "merchant_transaction_id",
                    models.CharField(
                        help_text="Merchant-provided reference number, should be unique for your transactions. Some receivers require this ID. This identifier is often used for reconciliation.",
                        max_length=255,
                        unique=True,
                        validators=[django.core.validators.MinLengthValidator(8)],
                        verbose_name="Merchant transaction ID",
                    ),
                ),
                (
                    "merchant_invoice_id",
                    models.CharField(
                        blank=True,
                        help_text="Merchant-provided invoice number, should be unique for your transactions. This identifier is not sent onwards.",
                        max_length=255,
                        null=True,
                        unique=True,
                        validators=[django.core.validators.MinLengthValidator(8)],
                        verbose_name="Merchant invoice ID",
                    ),
                ),
                (
                    "merchant_memo",
                    models.CharField(
                        blank=True,
                        help_text="Merchant-provided additional information. The information provided is not transaction processing relevant. It will appear in reporting only.",
                        max_length=255,
                        null=True,
                        validators=[django.core.validators.MinLengthValidator(8)],
                        verbose_name="Merchant memo",
                    ),
                ),
                (
                    "transaction_category",
                    models.CharField(
                        blank=True,
                        help_text="The category of the transaction. See docs for possible values:",
                        max_length=32,
                        null=True,
                        verbose_name="Transaction category",
                    ),
                ),
                (
                    "sandbox",
                    models.BooleanField(
                        help_text="Determine if this transaction uses the Test or Live host",
                        verbose_name="Sandbox",
                    ),
                ),
                (
                    "resource_path",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="VR Payment resource path can. Can be used to get the status",
                        max_length=512,
                        null=True,
                        verbose_name="VR Payment Resource Path",
                    ),
                ),
                (
                    "payment_id",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="The identifier of the payment request that can be used to reference the payment later",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator("[a-zA-Z0-9]{32}")
                        ],
                        verbose_name="VR Payment ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "VR Payment Basic Payment Checkout",
                "verbose_name_plural": "VR Payment Basic Checkouts",
            },
        ),
        migrations.CreateModel(
            name="VRPaymentCheckoutResponse",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created at"),
                ),
                (
                    "last_modified",
                    models.DateTimeField(auto_now=True, verbose_name="Last modified"),
                ),
                (
                    "http_status_code",
                    models.IntegerField(
                        help_text="response status code",
                        verbose_name="HTTP status code",
                    ),
                ),
                ("url", models.URLField(help_text="request url", verbose_name="URL")),
                (
                    "raw_headers",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="response header as json", verbose_name="Headers"
                    ),
                ),
                (
                    "raw_content",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="response content as json", verbose_name="Content"
                    ),
                ),
                (
                    "build_number",
                    models.CharField(
                        blank=True,
                        help_text="Useful for support purposes.",
                        max_length=255,
                        null=True,
                        verbose_name="API response build number",
                    ),
                ),
                (
                    "ndc",
                    models.CharField(
                        blank=True,
                        help_text="An internal unique identifier for the request.",
                        max_length=65,
                        null=True,
                        verbose_name="ndc",
                    ),
                ),
                (
                    "result_code",
                    models.CharField(
                        blank=True,
                        help_text="Status Code from VR Payment. see content for description",
                        max_length=11,
                        null=True,
                        verbose_name="VR Payment status code",
                    ),
                ),
                (
                    "result_description",
                    models.CharField(
                        blank=True,
                        help_text="A textual description explaining the result.code's meaning.",
                        max_length=255,
                        null=True,
                        verbose_name="Result Description",
                    ),
                ),
                (
                    "result_avs_response",
                    models.CharField(
                        blank=True,
                        help_text="Contains the AVS response returned by the acquirer. It may include one the following result:A = Address does match, zip code does not matchZ = Address does not match, zip code does matchN = Address and zip code do not matchU = Technical or logical error. AVS cannot be applied on card or address (not UK or US issuer), issuer is not available, etc.F = Address and Postal Code Matches",
                        max_length=1,
                        null=True,
                        verbose_name="Result avs response",
                    ),
                ),
                (
                    "result_cvv_response",
                    models.CharField(
                        blank=True,
                        help_text="Contains the CVV response returned by the acquirer.",
                        max_length=1,
                        null=True,
                        verbose_name="Result cvv response",
                    ),
                ),
                (
                    "result_details",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.",
                        null=True,
                        verbose_name="Result details",
                    ),
                ),
                (
                    "result_details_acquirer_response",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="Represents the acquirer original response code retrieved from the acquirer directly.",
                        null=True,
                        verbose_name="Result details - Acquirer response",
                    ),
                ),
                (
                    "card_bin",
                    models.CharField(
                        blank=True,
                        help_text="The first six digits of the card.number",
                        max_length=6,
                        null=True,
                        verbose_name="Card bin",
                    ),
                ),
                (
                    "card_holder",
                    models.CharField(
                        blank=True,
                        help_text="Holder of the credit card account",
                        max_length=6,
                        null=True,
                        verbose_name="Card holder",
                    ),
                ),
                (
                    "card_expiry_month",
                    models.CharField(
                        blank=True,
                        help_text="The expiry month of the card",
                        max_length=2,
                        null=True,
                        verbose_name="Card expiry month",
                    ),
                ),
                (
                    "card_expiry_year",
                    models.CharField(
                        blank=True,
                        help_text="The expiry year of the card",
                        max_length=4,
                        null=True,
                        verbose_name="Card expiry year",
                    ),
                ),
                (
                    "merchant_bank_account_holder",
                    models.CharField(
                        blank=True,
                        help_text="Holder of the merchant's bank account",
                        max_length=128,
                        null=True,
                        verbose_name="Merchant bank account holder",
                    ),
                ),
                (
                    "merchant_bank_account_nummber",
                    models.CharField(
                        blank=True,
                        help_text="The account number of the merchant's bank account. (IBAN for SEPA accounts)",
                        max_length=64,
                        null=True,
                        verbose_name="Merchant bank account nummber",
                    ),
                ),
                (
                    "merchant_bank_account_bic",
                    models.CharField(
                        blank=True,
                        help_text="The BIC (Bank Identifier Code (SWIFT)) number of the merchant's bank account.",
                        max_length=11,
                        null=True,
                        verbose_name="Merchant bank account bic",
                    ),
                ),
                (
                    "merchant_bank_account_country",
                    models.CharField(
                        blank=True,
                        help_text="The country code of the merchant's bank account (ISO 3166-1).",
                        max_length=2,
                        null=True,
                        verbose_name="Merchant bank account country",
                    ),
                ),
                (
                    "vr_pay_id",
                    models.CharField(
                        blank=True,
                        help_text="The VR Pay identifier of this object",
                        max_length=48,
                        null=True,
                        verbose_name="ID",
                    ),
                ),
                (
                    "reference_id",
                    models.CharField(
                        blank=True,
                        help_text="In case of referenced payment (e.g., Capture or Refund), this fields included to see which payment was referenced.",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                regex="[a-zA-Z0-9]{32}"
                            )
                        ],
                        verbose_name="ReferenceID",
                    ),
                ),
                (
                    "payment_brand",
                    models.CharField(
                        blank=True,
                        help_text="The payment brand of the request.",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                regex="[a-zA-Z0-9_] {1,32}"
                            )
                        ],
                        verbose_name="Payment brand",
                    ),
                ),
                (
                    "amount",
                    django_vr_pay.fields.VRPaymentDecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="The amount of the request",
                        max_digits=12,
                        null=True,
                        verbose_name="Amount",
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        default="EUR",
                        help_text="The ISO 4217 currency code of the payment request's amount",
                        max_length=3,
                        null=True,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "descriptor",
                    models.CharField(
                        blank=True,
                        help_text="The descriptor of the request.",
                        max_length=127,
                        null=True,
                        verbose_name="Descriptor",
                    ),
                ),
                (
                    "risk_score",
                    models.IntegerField(
                        blank=True,
                        help_text="Returns the score of the executed transaction risk checks. The value is a number from -99999 to +99999. Can be returned both for standalone risk requests and payment requests that include risk checks.",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(-99999),
                            django.core.validators.MaxValueValidator(99999),
                        ],
                        verbose_name="Risk score",
                    ),
                ),
                (
                    "other",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.",
                        null=True,
                        verbose_name="Other",
                    ),
                ),
                (
                    "basic_payment",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="checkout_response",
                        to="django_vr_pay.VRPaymentBasicPayment",
                        verbose_name="VR Payment Basic Payment Checkout",
                    ),
                ),
            ],
            options={
                "verbose_name": "VR Payment Checkout Reponse",
                "verbose_name_plural": "VR Payment Checkout Reponses",
            },
        ),
        migrations.CreateModel(
            name="VRPaymentBasicPaymentStatusResponse",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created at"),
                ),
                (
                    "last_modified",
                    models.DateTimeField(auto_now=True, verbose_name="Last modified"),
                ),
                (
                    "http_status_code",
                    models.IntegerField(
                        help_text="response status code",
                        verbose_name="HTTP status code",
                    ),
                ),
                ("url", models.URLField(help_text="request url", verbose_name="URL")),
                (
                    "raw_headers",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="response header as json", verbose_name="Headers"
                    ),
                ),
                (
                    "raw_content",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="response content as json", verbose_name="Content"
                    ),
                ),
                (
                    "build_number",
                    models.CharField(
                        blank=True,
                        help_text="Useful for support purposes.",
                        max_length=255,
                        null=True,
                        verbose_name="API response build number",
                    ),
                ),
                (
                    "ndc",
                    models.CharField(
                        blank=True,
                        help_text="An internal unique identifier for the request.",
                        max_length=65,
                        null=True,
                        verbose_name="ndc",
                    ),
                ),
                (
                    "result_code",
                    models.CharField(
                        blank=True,
                        help_text="Status Code from VR Payment. see content for description",
                        max_length=11,
                        null=True,
                        verbose_name="VR Payment status code",
                    ),
                ),
                (
                    "result_description",
                    models.CharField(
                        blank=True,
                        help_text="A textual description explaining the result.code's meaning.",
                        max_length=255,
                        null=True,
                        verbose_name="Result Description",
                    ),
                ),
                (
                    "result_avs_response",
                    models.CharField(
                        blank=True,
                        help_text="Contains the AVS response returned by the acquirer. It may include one the following result:A = Address does match, zip code does not matchZ = Address does not match, zip code does matchN = Address and zip code do not matchU = Technical or logical error. AVS cannot be applied on card or address (not UK or US issuer), issuer is not available, etc.F = Address and Postal Code Matches",
                        max_length=1,
                        null=True,
                        verbose_name="Result avs response",
                    ),
                ),
                (
                    "result_cvv_response",
                    models.CharField(
                        blank=True,
                        help_text="Contains the CVV response returned by the acquirer.",
                        max_length=1,
                        null=True,
                        verbose_name="Result cvv response",
                    ),
                ),
                (
                    "result_details",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.",
                        null=True,
                        verbose_name="Result details",
                    ),
                ),
                (
                    "result_details_acquirer_response",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="Represents the acquirer original response code retrieved from the acquirer directly.",
                        null=True,
                        verbose_name="Result details - Acquirer response",
                    ),
                ),
                (
                    "card_bin",
                    models.CharField(
                        blank=True,
                        help_text="The first six digits of the card.number",
                        max_length=6,
                        null=True,
                        verbose_name="Card bin",
                    ),
                ),
                (
                    "card_holder",
                    models.CharField(
                        blank=True,
                        help_text="Holder of the credit card account",
                        max_length=6,
                        null=True,
                        verbose_name="Card holder",
                    ),
                ),
                (
                    "card_expiry_month",
                    models.CharField(
                        blank=True,
                        help_text="The expiry month of the card",
                        max_length=2,
                        null=True,
                        verbose_name="Card expiry month",
                    ),
                ),
                (
                    "card_expiry_year",
                    models.CharField(
                        blank=True,
                        help_text="The expiry year of the card",
                        max_length=4,
                        null=True,
                        verbose_name="Card expiry year",
                    ),
                ),
                (
                    "merchant_bank_account_holder",
                    models.CharField(
                        blank=True,
                        help_text="Holder of the merchant's bank account",
                        max_length=128,
                        null=True,
                        verbose_name="Merchant bank account holder",
                    ),
                ),
                (
                    "merchant_bank_account_nummber",
                    models.CharField(
                        blank=True,
                        help_text="The account number of the merchant's bank account. (IBAN for SEPA accounts)",
                        max_length=64,
                        null=True,
                        verbose_name="Merchant bank account nummber",
                    ),
                ),
                (
                    "merchant_bank_account_bic",
                    models.CharField(
                        blank=True,
                        help_text="The BIC (Bank Identifier Code (SWIFT)) number of the merchant's bank account.",
                        max_length=11,
                        null=True,
                        verbose_name="Merchant bank account bic",
                    ),
                ),
                (
                    "merchant_bank_account_country",
                    models.CharField(
                        blank=True,
                        help_text="The country code of the merchant's bank account (ISO 3166-1).",
                        max_length=2,
                        null=True,
                        verbose_name="Merchant bank account country",
                    ),
                ),
                (
                    "vr_pay_id",
                    models.CharField(
                        blank=True,
                        help_text="The VR Pay identifier of this object",
                        max_length=48,
                        null=True,
                        verbose_name="ID",
                    ),
                ),
                (
                    "reference_id",
                    models.CharField(
                        blank=True,
                        help_text="In case of referenced payment (e.g., Capture or Refund), this fields included to see which payment was referenced.",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                regex="[a-zA-Z0-9]{32}"
                            )
                        ],
                        verbose_name="ReferenceID",
                    ),
                ),
                (
                    "payment_brand",
                    models.CharField(
                        blank=True,
                        help_text="The payment brand of the request.",
                        max_length=32,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                regex="[a-zA-Z0-9_] {1,32}"
                            )
                        ],
                        verbose_name="Payment brand",
                    ),
                ),
                (
                    "amount",
                    django_vr_pay.fields.VRPaymentDecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="The amount of the request",
                        max_digits=12,
                        null=True,
                        verbose_name="Amount",
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        default="EUR",
                        help_text="The ISO 4217 currency code of the payment request's amount",
                        max_length=3,
                        null=True,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "descriptor",
                    models.CharField(
                        blank=True,
                        help_text="The descriptor of the request.",
                        max_length=127,
                        null=True,
                        verbose_name="Descriptor",
                    ),
                ),
                (
                    "risk_score",
                    models.IntegerField(
                        blank=True,
                        help_text="Returns the score of the executed transaction risk checks. The value is a number from -99999 to +99999. Can be returned both for standalone risk requests and payment requests that include risk checks.",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(-99999),
                            django.core.validators.MaxValueValidator(99999),
                        ],
                        verbose_name="Risk score",
                    ),
                ),
                (
                    "other",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.",
                        null=True,
                        verbose_name="Other",
                    ),
                ),
                (
                    "basic_payment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="payment_responses",
                        to="django_vr_pay.VRPaymentBasicPayment",
                        verbose_name="VR Payment Basic Checkouts",
                    ),
                ),
            ],
            options={
                "verbose_name": "VR Payment Payment Response",
                "verbose_name_plural": "VR Payment Payment Responses",
            },
        ),
        migrations.AddIndex(
            model_name="vrpaymentbasicpayment",
            index=models.Index(
                fields=["merchant_transaction_id"],
                name="django_vr_p_merchan_fe8f32_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="vrpaymentbasicpayment",
            index=models.Index(
                fields=["merchant_invoice_id"], name="django_vr_p_merchan_106324_idx"
            ),
        ),
    ]
